#include "SybaseServer.h"
#include "SybaseClass.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

CS_CONTEXT *SybaseContext;
CS_CONNECTION *SybaseConnection;

SybaseServer::SybaseServer(void)
	:connected(false), msql(-1)
	{
	}

SybaseServer::~SybaseServer(void)
	{
	}

QueryResult SybaseServer::connect(QString addr, QString user, QString passwd, int port )   {
	QueryResult res;
	CS_RETCODE ret;
	SybaseClass Sybase;
  	const char *caddr = (const char *)addr;
	Sybase.SybaseInit(&SybaseContext);
  	ret = Sybase.SybaseConnect(SybaseContext, &SybaseConnection, "Kmsql", (CS_CHAR *) (const char *)user, (CS_CHAR *) (const char *) passwd, (CS_CHAR *) (const char *)addr);

  	if(ret != -1)
  	{
		res.type = QueryResult::Ok;
		connected = true;
	}
	else
	{
		res.type = QueryResult::Error;
		res.message = "Connection pb " ;
	}
	return res;
}


QueryResult SybaseServer::getColumns(QString, QString)
  {
	QueryResult res;  
	res.type = QueryResult::Unsuported;
	return res;
  }


void SybaseServer::close(void)
  {
	CS_RETCODE status;
	SybaseClass Sybase;
   	//printf("begin close\n");
   	Sybase.Sybase_con_cleanup(SybaseConnection, status);
	Sybase.CtxCleanup(SybaseContext,status);
  	connected = false;
  }

QueryResult SybaseServer::selectBase(QString baseName)
  {
	//printf("begin select base\n");
	QueryResult 		res;
	CS_RETCODE 		ret;
	SybaseClass 		Sybase;

	char *cbaseName = strdup(baseName);
	
	ret = Sybase.UseDb(SybaseConnection, cbaseName);
   	if(ret != -1)
	{
	 res.type = QueryResult::Ok;
	}
	else
	{
	res.type = QueryResult::Error;
	res.message = "Sybase Error selecting base";
    }
	delete cbaseName;
	return res;
  }

QueryResult SybaseServer::query(QString query)
  {
 	CS_COMMAND		*cmd;
	CS_INT			res_type;	/* result type from ct_results */
   	CS_RETCODE		retcode;
	CS_INT			num_cols;
	CS_INT			i;
	CS_INT			j;
	CS_INT			row_count = 0;
	CS_INT			rows_read;
	CS_INT			disp_len;
     	QueryResult 		queryRes;
	SybaseClass 		Sybase;

  	char *cquery = strdup(query);

	//printf("%s\n", cquery);
 	/*
	** Allocate a command handle to send the compute query with
	*/
        if ((retcode = ct_cmd_alloc(SybaseConnection, &cmd)) != CS_SUCCEED)
        {
                Sybase.Error("query: ct_cmd_alloc() failed");
        }

	/*
	** Define a language command that contains a compute clause.  SELECT
	** is a select statment defined in the header file.
	*/
	retcode = ct_command(cmd, CS_LANG_CMD, cquery, CS_NULLTERM, CS_UNUSED);
	if (retcode != CS_SUCCEED)
	{
                Sybase.Error("query: ct_command() failed");
	}

	/*
	** Send the command to the server 
	*/
	if (ct_send(cmd) != CS_SUCCEED)
	{
                Sybase.Error("query: ct_send() failed");
	}
					
	/*
	** Process the results.  Loop while ct_results() returns CS_SUCCEED.
	*/
	while ((retcode = ct_results(cmd, &res_type)) == CS_SUCCEED)
	{
		switch ((int)res_type)
		{
		case CS_CMD_SUCCEED:
			/*
			** This means no rows were returned.  
			*/
			break;

		case CS_CMD_DONE:
			/*
			** This means we're done with one result set.
			*/
   			break;

		case CS_CMD_FAIL:
			/*
			** This means that the server encountered an error while
			** processing our command.
			*/
			Sybase.Error("query: ct_results() returned CMD_FAIL");
			break;

		case CS_ROW_RESULT:
			//printf("use fetch3 \n");
			retcode = Sybase.Fetch_data3(cmd, &queryRes, &row_count);
			if (retcode != CS_SUCCEED)
			{
				Sybase.Error("query: fetch_data3() failed");
			}
			break;

		case CS_COMPUTE_RESULT:
			//printf("use compute \n");
			retcode = Sybase.FetchComputeResults(cmd, &queryRes);
			if (retcode != CS_SUCCEED)
			{
				Sybase.Error("query: FetchComputeResults() failed");
			}
			break;

		default:
			/*CS_COMPUTE_RESULT
			** We got an unexpected result type.
			*/
			Sybase.Error("query: ct_results() returned unexpected result type");
		}
	}

	/*
	** We're done processing results. Let's check the
	** return value of ct_results() to see if everything
	** went ok.
	*/
	switch ((int)retcode)
	{
		case CS_END_RESULTS:
			/*
			** Everything went fine.
			*/
			break;

		case CS_FAIL:
			/*
			** Something went wrong.
			*/
			Sybase.Error("query: ct_results() failed");

		default:
			/*
			** We got an unexpected return value.
			*/
			Sybase.Error("query: ct_results() returned unexpected result code");
	}

	/*
	** Drop our command structure
	*/
	if ((retcode = ct_cmd_drop(cmd)) != CS_SUCCEED)
	{
                Sybase.Error("query: ct_cmd_drop() failed");
	}


  // On retourne
  queryRes.type = QueryResult::Set;
  QString s;
  s.sprintf("Query OK, %d %s in set.", row_count, row_count>1 ? "rows" : "row");
  queryRes.message = s;
  delete cquery;
  return queryRes;
  }

QueryResult SybaseServer::getBases(void)
  {
	QueryResult 		queryRes;
	SybaseClass		Sybase;	
	QString base = "master";
	
	queryRes = Sybase.getObjects(base, "select name from sysdatabases" );
	return queryRes;
  }



QueryResult SybaseServer::getTables(QString baseName, QString wild)
  {
	QueryResult 		queryRes;
	SybaseClass		Sybase;	
	char myquery[] = "select user_name(uid) + \".\" + O.name from sysobjects O, sysindexes I, syssegments S \
                    where O.id=I.id and I.indid in (0,1) and I.segment = S.segment and O.type!='S' ";	
	char *cbaseName = strdup(baseName);
	queryRes = Sybase.getObjects(cbaseName, myquery );
	return queryRes;
  }

QueryResult  getStoredProcedures(QString baseName)
{
	QueryResult		queryRes;
	SybaseClass		Sybase;	
	char myquery[] ="SELECT user_name(uid) + '.' +O.name FROM sysobjects O WHERE  O.type='P'";
	
	char *cbaseName = strdup(baseName);
	queryRes = Sybase.getObjects(cbaseName, myquery );

	return queryRes;
}

QueryResult  getIndexes(QString baseName)
{
	QueryResult		queryRes;
	SybaseClass		Sybase;	
	char myquery[] ="";

	queryRes = Sybase.getObjects(baseName, myquery );
	return queryRes;
}

QueryResult SybaseServer::shutdown(void)
  {
	//printf("begin sutdown");
	QueryResult queryRes;
	queryRes.type = QueryResult::Error;
	queryRes.message = "Unsupported by Sybase";
	return queryRes;
  }

 QueryResult SybaseServer::reload(void)
  {
	//printf("begin reload\n");
	QueryResult queryRes;
	queryRes.type = QueryResult::Error;
	queryRes.message = "Unsupported by Sybase";
	return queryRes;
  }

QString SybaseServer::serverInfo(void)
  {
	QueryResult	queryRes;
	SybaseClass	Sybase;
	char myquery[] ="select attribute_value from sybsystemprocs.dbo.spt_server_info \
			 where attribute_id = 2";
	QString baseName = "sybsystemprocs";

	queryRes = Sybase.getObjects(baseName, myquery );
	return queryRes.result.at(0);
	//return "No info available for Sybase server";
  }

QueryResult SybaseServer::createBase(QString name)
  {
  	QueryResult	queryRes;
	SybaseClass	Sybase;

  	char *cbaseName = strdup(name);
	Sybase.CreateDb(SybaseConnection, cbaseName);
  	return queryRes;
  }

QueryResult SybaseServer::dropBase(QString name)
  {
	QueryResult	queryRes;
	SybaseClass	Sybase;

	char *cbaseName = strdup(name);
	Sybase.RemoveDb(SybaseConnection, cbaseName);
  	return queryRes;
  }

IObject *SybaseServer::queryInterface(UIID id)
  {
    	//printf("query interface \n");
	if(id == UIID_SQLSERVER)
		return this;
	else
    	return 0;
  }


