
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SybaseClass.h"
#include "SybaseCommon.h"
extern CS_CONTEXT *SybaseContext;
extern CS_CONNECTION *SybaseConnection;

/*****************************************************************************
** 
** callback functions 
** 
*****************************************************************************/

/*
** ClientmsgCb()
**
** Type of function:
** 	example program client message handler
**
** Purpose:
** 	Installed as a callback into Open Client.
**
** Returns:
** 	CS_SUCCEED
**
** Side Effects:
** 	None
*/

CS_RETCODE CS_PUBLIC ClientmsgCb(CS_CONTEXT *context, CS_CONNECTION *connection, CS_CLIENTMSG *errmsg)
{
	fprintf(EX_ERROR_OUT, "\nOpen Client Message:\n");
	fprintf(EX_ERROR_OUT, "Message number: LAYER = (%ld) ORIGIN = (%ld) ",
		CS_LAYER(errmsg->msgnumber), CS_ORIGIN(errmsg->msgnumber));
	fprintf(EX_ERROR_OUT, "SEVERITY = (%ld) NUMBER = (%ld)\n",
		CS_SEVERITY(errmsg->msgnumber), CS_NUMBER(errmsg->msgnumber));
	fprintf(EX_ERROR_OUT, "Message String: %s\n", errmsg->msgstring);
	if (errmsg->osstringlen > 0)
	{
		fprintf(EX_ERROR_OUT, "Operating System Error: %s\n",
			errmsg->osstring);
	}
	fflush(EX_ERROR_OUT);

	return CS_SUCCEED;
}

/*
** ServermsgCb()
**
** Type of function:
** 	example program server message handler
**
** Purpose:
** 	Installed as a callback into Open Client.
**
** Returns:
** 	CS_SUCCEED
**
** Side Effects:
** 	None
*/
CS_RETCODE CS_PUBLIC ServermsgCb(CS_CONTEXT *context, CS_CONNECTION *connection, CS_SERVERMSG *srvmsg)
{
	fprintf(EX_ERROR_OUT, "\nServer message:\n");
	fprintf(EX_ERROR_OUT, "Message number: %ld, Severity %ld, ",
		srvmsg->msgnumber, srvmsg->severity);
	fprintf(EX_ERROR_OUT, "State %ld, Line %ld\n",
		srvmsg->state, srvmsg->line);
	
	if (srvmsg->svrnlen > 0)
	{
		fprintf(EX_ERROR_OUT, "Server '%s'\n", srvmsg->svrname);
	}
	
	if (srvmsg->proclen > 0)
	{
		fprintf(EX_ERROR_OUT, " Procedure '%s'\n", srvmsg->proc);
	}

	fprintf(EX_ERROR_OUT, "Message String: %s\n", srvmsg->text);
	fflush(EX_ERROR_OUT);

	return CS_SUCCEED;
}

